import debounce from 'lodash/debounce';
import validate from '@meta-awesome/validators';
import validateRequired from '@meta-awesome/validators/src/required';

export default {
  data() {
    return {
      rules: [],
      validator: {
        touched: false,
        isValid: true,
        error: false,
        errorText: ''
      }
    };
  },

  props: {
    /**
     * String containing the custom error
     * */
    error: {
      type: String,
      required: false,
      default: null
    }
  },

  methods: {
    validateValue: debounce(function(value) {
      const response = validate(value, ...this.computedRules);
      this.validator = {
        ...this.validator,
        ...response,
        error: !response.isValid && this.validator.touched
      };

      this.$emit('validation', this.validator);
    }, 100),

    setTouched(value) {
      if (!this.validator.touched) {
        this.validator = {
          ...this.validator,
          touched: true
        };

        this.validateValue(value);
      }
    }
  },

  computed: {
    computedRules() {
      const rules = [...this.rules];

      if (this.required) {
        rules.push(validateRequired);
      }

      return rules;
    },
    hasError() {
      return !!this.error || this.validator.error;
    },
    errorText() {
      return this.error || (this.validator.error && this.validator.errorText);
    }
  }
};
